/*

   []----------------------------------------------------------------------[]
   |                               MOUSE.C                                  |
   |                                                                        |
   |                  Funcoes para manipulacao do mouse.                    |
   |                                                                        |
   |                  (C) 1995, Lenimar Nunes de Andrade                    |
   []----------------------------------------------------------------------[]

*/

#include <dos.h>

#define MOUSE        0x33
#define BOTAO_ESQUERDO  0
#define BOTAO_DIREITO   1
#define BOTAO_CENTRAL   2
#define NAO_PRESSIONADO 3

int mouse;        /* Verifica se tem suporte ao mouse */
extern int TempoDeDemoraDoMouse;

#define DEMORA delay(TempoDeDemoraDoMouse)

/* Definicoes dos formatos dos indicadores do mouse em modo grafico */

unsigned int Glove[] = {
 0xF3FF, 0xEDFF, 0xEDFF, 0xEDFF, 0xEDFF, 0xEC49, 0xEDB6, 0x8DB6,
 0x6DB6, 0x6FFE, 0x6FFE, 0x7FFE, 0x7FFE, 0xBFFD, 0xBFFD, 0xC003,
  0xC00, 0x1200, 0x1200, 0x1200, 0x1200, 0x13B6, 0x1249, 0x7249,
 0x9249, 0x9001, 0x9001, 0x8001, 0x8001, 0x4002, 0x4002, 0x3FFC
};

unsigned int Cross[] = {
 0xFFFF, 0xF80F, 0xF777, 0xEF7B, 0xDF7D, 0xBF7E, 0xBF7E, 0xBFFE,
 0x81C0, 0xBFFE, 0xBF7E, 0xBF7E, 0xDF7D, 0xEF7B, 0xF777, 0xF80F,
    0x0,  0x7F0,  0x888, 0x1084, 0x2082, 0x4081, 0x4081, 0x4001,
 0x7E3F, 0x4001, 0x4081, 0x4081, 0x2082, 0x1084,  0x888,  0x7F0
};

#define X_GLOVE  5     /* Definicoes dos pontos fortes */
#define Y_GLOVE  1     /* dos indicadores do mouse     */
#define X_CROSS  8
#define Y_CROSS  8

/* ------------------------------------------------------------------------ */

int HaSuporteAoMouse(void) {

  /* Verifica se ha' suporte ao mouse. Retorna a quantidade de botoes.
     O indicador do mouse e' ajustado escondido para o centro da tela. */

  union REGS regs;

  regs.x.ax = 0;
  int86(MOUSE, &regs, &regs);

  if (regs.x.ax == 0xFFFF)
    return regs.x.bx; /* Ha' suporte ao mouse. Retorna a
			 quantidade de botoes do  mouse. */
  else
    return 0;         /* Nao ha' suporte ao mouse */
}

/* ------------------------------------------------------------------------ */

void MostraMouse(void) {

  /* Mostra o indicador do mouse */

  union REGS regs;

  regs.x.ax = 1;
  int86(MOUSE, &regs, &regs);
}

/* ------------------------------------------------------------------------ */

void EscondeMouse(void) {

/* Esconde o indicador do mouse. O acionador continua seguindo a posicao
   do mouse. */

  union REGS regs;

  regs.x.ax = 2;
  int86(MOUSE, &regs, &regs);
}

/* ------------------------------------------------------------------------ */

int PosicaoDoMouse(int *pos_x, int *pos_y) {

/* Obtem a posicao atual do  indicador  do  mouse.  As  coordenadas sao
   devolvidas em pixels independentemente do modo de apresentacao atual.
   A posicao (0, 0) e' o canto superior esquerdo da tela. */

  union REGS regs;

  regs.x.ax = 3;
  int86(MOUSE, &regs, &regs);
  *pos_x = regs.x.cx;
  *pos_y = regs.x.dx;
  if (regs.x.bx & 1) return BOTAO_ESQUERDO;   /* botao esquerdo pressionado */
  else if (regs.x.bx &2) return BOTAO_DIREITO; /* botao direito pressionado */
  else if (regs.x.bx &4) return BOTAO_CENTRAL; /* botao central pressionado */
  else return NAO_PRESSIONADO;                 /* nenhum  botao pressionado */
}

/* ------------------------------------------------------------------------ */

void AjustaPosicaoDoMouse(int pos_x, int pos_y) {

/* As coordenadas sao especificadas em pixels.  A  posicao  (0, 0)  e'  o
   canto superior esquerdo da tela. Se necessario, a posicao e'  ajustada
   para permanecer dentro dos limites horizontal e vertical especificados. */

  union REGS regs;

  regs.x.ax = 4;
  regs.x.cx = pos_x;
  regs.x.dx = pos_y;
  int86(MOUSE, &regs, &regs);
}

/* ------------------------------------------------------------------------ */

void AjustaLimitesVerticais(int liminf, int limsup) {

  /* Ajusta limites verticais do mouse em pixels */

  union REGS regs;

  regs.x.ax = 8;
  regs.x.cx = liminf;
  regs.x.dx = limsup;
  int86(MOUSE, &regs, &regs);
}

/* ------------------------------------------------------------------------ */

void AjustaLimitesHorizontais(int liminf, int limsup) {

  /* Ajusta limites horizontais do mouse em pixels */

  union REGS regs;

  regs.x.ax = 7;
  regs.x.cx = liminf;
  regs.x.dx = limsup;
  int86(MOUSE, &regs, &regs);
}

/* ------------------------------------------------------------------------ */

void AjustaFormaDoIndicadorGrafico(int desl_esquerda, int desl_topo,
				   long formato) {

/*  Define forma, cor e ponto forte do indicador do mouse em modo grafico.
    desl_esquerda e' o deslocamento do ponto forte a partir  da  esquerda,
    desl_topo  e'  o  deslocamento  do  ponto  forte  a  partir  do  topo,
    formato deve estar relacionado com o endereco do inicio  da matriz de
    imagem  do indicador. */

  struct REGPACK regs;

  regs.r_ax = 9;
  regs.r_bx = desl_esquerda;
  regs.r_cx = desl_topo;
  regs.r_es = formato >> 16;
  regs.r_dx = formato & 0xFFFF;
  intr(MOUSE, &regs);
}

/* ------------------------------------------------------------------------ */

/*** FIM DE MOUSE.C ***/

