/*** SPLINES.C ***/

typedef struct{ float x, y, z; } t_ponto;

#define MAX_PONTOS 10
t_ponto p1[MAX_PONTOS][MAX_PONTOS], p2[MAX_PONTOS][MAX_PONTOS];
int m, n;

#define X_ELEM(M, i, j) (M[i][j]).x
#define Y_ELEM(M, i, j) (M[i][j]).y
#define Z_ELEM(M, i, j) (M[i][j]).z

#define DEF_PONTO(P, M, N, X, Y, Z)   P[M][N].x=X; P[M][N].y=Y; P[M][N].z=Z

extern unsigned _stklen = 20000;

#define MAX_NOS 100
float x1[MAX_NOS], x2[MAX_NOS];

t_ponto P[100];
int m, n, k1, k2, A, B, TAM;

/* ---------------------------------------------------------------------- */

void  VetorDeNosUniforme(int k, int n, float X[]) {

  int i;

  for (i = 1; i <= k; i++) X[i] = 0.0;
  for (i = k + 1; i <= n + 1; i++) X[i] = ((float) (i - k))/(n - k + 2.0);
  for (i = n + 2; i <= n + k + 1; i++) X[i] = 1.0; /* n - k + 2; */
}

/* ---------------------------------------------------------------------- */

float N(float x[], int i, int k, float t) {

/* B-spline N(i, k, t). Devem ter sido definido antes os ns x[j] com
   1  j  i + k + 1   e   x[j]  x[j + 1]  para todo j */

  float soma = 0;

  if (k == 1) {
    if (x[i] <= t && t < x[i + 1])
      return 1;
    else
      return 0;
  }

  if (k > 1) {
    if (x[i + k - 1] != x[i] && t != x[i])
      soma += (t - x[i])/(x[i + k - 1] - x[i])*N(x, i, k - 1, t);
    if (x[i + k] != x[i + 1] && t != x[i + k])
      soma += (x[i + k] - t)/(x[i + k] - x[i + 1])*N(x, i + 1, k - 1, t);
  }
  return soma;
}

/* ---------------------------------------------------------------------- */

float X_Spline(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n,
	 float x1[], float x2[], int k1, int k2, float u, float v) {

/* Calcula a parametrizacao da coordenada X da superficie de Bezier com
   malha de controle dada pelos m.n pontos P(). u, v devem estar no
   intervalo [0, 1] */

  float soma = 0;
  int i, j;

  for (i = 1; i <= m + 1; i++)
    for (j = 1; j <= n + 1; j++)
      soma += X_ELEM(P, i, j)*N(x1, i, k1, u)*N(x2, j, k2, v);

  return soma;
}

/* ------------------------------------------------------------------------ */

float Y_Spline(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n,
	 float x1[], float x2[], int k1, int k2, float u, float v) {

/* Calcula a parametrizacao da coordenada Y da superficie de Bezier com
   malha de controle dada pelos m.n pontos P(). u, v devem estar no
   intervalo [0, 1] */

  float soma = 0;
  int i, j;

  for (i = 1; i <= m + 1; i++)
    for (j = 1; j <= n + 1; j++)
      soma += Y_ELEM(P, i, j)*N(x1, i, k1, u)*N(x2, j, k2, v);

  return soma;
}

/* ------------------------------------------------------------------------ */

float Z_Spline(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n,
	 float x1[], float x2[], int k1, int k2, float u, float v) {

/* Calcula a parametrizacao da coordenada Z da superficie de Bezier com
   malha de controle dada pelos m.n pontos P(). u, v devem estar no
   intervalo [0, 1] */

  float soma = 0;
  int i, j;

  for (i = 1; i <= m + 1; i++)
    for (j = 1; j <= n + 1; j++)
      soma += Z_ELEM(P, i, j)*N(x1, i, k1, u)*N(x2, j, k2, v);

  return soma;
}

/* ------------------------------------------------------------------------ */

/*** FIM DE SPLINES.C ***/

