
/*** SP_DEMO.C - Desenha uma superficie B-spline ***/

#include <math.h>
#include <graphics.h>
#include <conio.h>
#include <stdio.h>
#include <string.h>

#define float long double
#define CAMINHO_BGI   ""
int cor1 = BLUE, cor2 = LIGHTRED;

#include "splines.c"
#include "graficos.c"

float f1(float u, float v) { return X_Spline(p1, m, n, x1, x2, k1, k2, u, v); }
float f2(float u, float v) { return Y_Spline(p1, m, n, x1, x2, k1, k2, u, v); }
float f3(float u, float v) { return Z_Spline(p1, m, n, x1, x2, k1, k2, u, v); }

/* ------------------------------------------------------------------------ */

void MarcaPontoControle(float x, float y, float z, int cor) {

  float X, Y;

  ProjetaPonto(x, y, z, &X, &Y);
  putpixel((int) (A + X*TAM), (int) (B - Y*TAM), cor);
  putpixel((int) (A + X*TAM + 1), (int) (B - Y*TAM + 1), cor);
  putpixel((int) (A + X*TAM + 1), (int) (B - Y*TAM - 1), cor);
  putpixel((int) (A + X*TAM - 1), (int) (B - Y*TAM + 1), cor);
  putpixel((int) (A + X*TAM - 1), (int) (B - Y*TAM - 1), cor);
}

/* ------------------------------------------------------------------------ */

void MarcaPontosDeControle(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n) {

  int i, j;

  for (i = 1; i <= m + 1; i++)
    for (j = 1; j <= n + 1; j++)
      MarcaPontoControle(X_ELEM(P, i, j), Y_ELEM(P, i, j), Z_ELEM(P, i, j),
		  cor2);

}

/* ------------------------------------------------------------------------ */

void ImprimePontosCtrl(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n) {

  int i, j;

  printf("\n\nOs ultimos pontos de controle mostrados sao:\n");
  for (i = 1; i <= m + 1; i++)
    for (j = 1; j <= n + 1; j++)
      printf("\n    P[%d][%d] = (%5.1Lf, %5.1Lf, %5.1Lf)",
		   i, j,  X_ELEM(P, i, j), Y_ELEM(P, i, j), Z_ELEM(P, i, j));
  printf("\n\n");
}

/* ------------------------------------------------------------------------ */

int  MostraNumeros(t_ponto P[MAX_PONTOS][MAX_PONTOS], int m, int n) {

  int i, j, cont = 0;
  float X, Y;
  char mens[10];

  for (i = 1; i <= m + 1; i++)
    for (j = 1; j <= n + 1; j++) {
      cont++;

      ProjetaPonto(X_ELEM(P, i, j), Y_ELEM(P, i, j), Z_ELEM(P, i, j), &X, &Y);
      sprintf(mens, "%d", cont);
      outtextxy((int) (A + X*TAM), (int) (B - Y*TAM), mens);
  }
  outtextxy(1, getmaxy() - 3*textheight("["),
			 " \x10 Pressione Ctrl-Fn (\x19) ou Alt-Fn (\x18) "
			 "para mover o ponto de controle n");
  return 0;
}

/* ------------------------------------------------------------------------ */

void MensagemInicial(void) {

  printf("\n\n*** ESBOO DE UMA SUPERFICIE B-SPLINE COM 9 PONTOS DE CONTROLE ***");
  printf("\n\n  ESC: encerra o programa");
  printf("\n  PgUp: aproxima o grafico         PgDn: afasta o grafico");
  printf("\n  Teclas de setas: movimentam a origem (0, 0, 0) na tela");
  printf("\n  F1: mostra eixos                 F3: muda cor do grafico");
  printf("\n  F2: mostra numeros dos pontos de controle");
  printf("\n  F4: muda cor dos pontos de controle");
  printf("\n  Ctrl-Fn: desloca para baixo o n-esimo ponto de controle");
  printf("\n  Alt-Fn: desloca para cima o n-esimo ponto de controle");
  printf("\n\nPressione qualquer tecla para continuar\n\n");
  getch();

}

/* ------------------------------------------------------------------------ */

void main(void) {

  int eixos = 1, numeros = 1;
  char ch;

  nu = 30, nv = 30, inc = 0.06, TAM = 65;
  IniciaModoGrafico();
  DeterminaPlanoDeProjecao();
  A = getmaxx()/2; B = getmaxy()/2;

  k1 = k2 = 3;
  m = n = 4;
  VetorDeNosUniforme(k1, m, x1);
  VetorDeNosUniforme(k2, n, x2);

  DEF_PONTO(p1, 1, 1, -5, -5, 1);
  DEF_PONTO(p1, 1, 2, -5, -3, 2);
  DEF_PONTO(p1, 1, 3, -5, -1, 1);
  DEF_PONTO(p1, 1, 4, -5, 1, 1);
  DEF_PONTO(p1, 1, 5, -5, 3, 3);

  DEF_PONTO(p1, 2, 1, -3, -5, 2);
  DEF_PONTO(p1, 2, 2, -3, -3, 3);
  DEF_PONTO(p1, 2, 3, -3, -1, 1);
  DEF_PONTO(p1, 2, 4, -3, 1, 1);
  DEF_PONTO(p1, 2, 5, -3, 3, 1);

  DEF_PONTO(p1, 3, 1, -1, -5, 1);
  DEF_PONTO(p1, 3, 2, -1, -3, 1);
  DEF_PONTO(p1, 3, 3, -1, -1, 1);
  DEF_PONTO(p1, 3, 4, -1, 1, 3);
  DEF_PONTO(p1, 3, 5, -1, 3, 2);

  DEF_PONTO(p1, 4, 1, 1, -5, 1);
  DEF_PONTO(p1, 4, 2, 1, -3, 3);
  DEF_PONTO(p1, 4, 3, 1, -1, 3);
  DEF_PONTO(p1, 4, 4, 1, 1, 1);
  DEF_PONTO(p1, 4, 5, 1, 3, 5);

  DEF_PONTO(p1, 5, 1, 3, -5, 1);
  DEF_PONTO(p1, 5, 2, 3, -3, -1);
  DEF_PONTO(p1, 5, 3, 3, -1, 0);
  DEF_PONTO(p1, 5, 4, 3, 1, -1);
  DEF_PONTO(p1, 5, 5, 3, 3, 1);

  setcolor(cor1);
  DesenhaSuperficie2(f1, f2, f3, 0.0001, 0.9999, 0.0001, 0.999);
  if (eixos) {setcolor(WHITE); DesenhaEixos(3, GREEN); }
  MarcaPontosDeControle(p1, m, n);

  ch = getch();
  closegraph();
}

/*** FIM DE SP_DEMO.C ***/

